import { publicKey } from './PublicKey.mjs';
import { InterfaceImplementationMissingError } from './errors/SdkError.mjs';

const getProgramAddressWithFallback = (context, name, address) => {
  const key = publicKey(address);
  if (!context.programs) return key;
  try {
    return context.programs.get(name).address;
  } catch (error) {
    return key;
  }
};
class NullProgramRepository {
  get() {
    throw new InterfaceImplementationMissingError('ProgramRepositoryInterface', 'programs');
  }
  all() {
    throw new InterfaceImplementationMissingError('ProgramRepositoryInterface', 'programs');
  }
  add() {
    throw new InterfaceImplementationMissingError('ProgramRepositoryInterface', 'programs');
  }
  resolveError() {
    throw new InterfaceImplementationMissingError('ProgramRepositoryInterface', 'programs');
  }
}

export { NullProgramRepository, getProgramAddressWithFallback };
//# sourceMappingURL=ProgramRepositoryInterface.mjs.map
